@@ -4,7 +4,7 @@ from xml.dom import minidom, getDOMImplementation
 
 from intermine.util import openAnything, ReadableException
 from intermine.pathfeatures import PathDescription, Join, SortOrder, SortOrderList
-from intermine.model import Column, Class, Model, Reference
+from intermine.model import Column, Class, Model, Reference, ConstraintNode
 import constraints
 
 """
@@ -816,6 +816,7 @@ class Query(object):
           - Check that SubClassConstraints have a correct subclass relationship
           - Check that LoopConstraints have a valid loopPath, of a compatible type
           - Check that ListConstraints refer to an object
+          - Don't even try to check RangeConstraints: these have variable semantics
 
         @param cons: The constraints to check (defaults to all constraints on the query)
 
@@ -826,7 +827,9 @@ class Query(object):
         if cons is None: cons = self.constraints
         for con in cons:
             pathA = self.model.make_path(con.path, self.get_subclass_dict())
-            if isinstance(con, constraints.TernaryConstraint):
+            if isinstance(con, constraints.RangeConstraint):
+                pass # No verification done on these, beyond checking its path, of course.
+            elif isinstance(con, constraints.TernaryConstraint):
                 if pathA.get_class() is None:
                     raise ConstraintError("'" + str(pathA) + "' does not represent a class, or a reference to a class")
             elif isinstance(con, constraints.BinaryConstraint) or isinstance(con, constraints.MultiConstraint):
@@ -1513,10 +1516,17 @@ class Query(object):
 
     def to_query(self):
         """
-        Return the query to allow equivalent treatment of lists and queries.
+        Implementation of trait that allows use of these objects as queries (casting).
         """
         return self
 
+    def make_list_constraint(self, path, op):
+        """
+        Implementation of trait that allows use of these objects in list constraints
+        """
+        l = self.service.create_list(self)
+        return ConstraintNode(path, op, l.name)
+
     def to_query_params(self):
         """
         Returns the parameters to be passed to the webservice
